<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Teacher_Discipline_Edit extends Controller
{
    /** @var  User */
    private $user;

    /** @var  Model_Discipline */
    private $discipline;

    /** @var  Twig */
    private $twig;

    const twigSource = 'teacher/discipline/edit/';


    public function before() {
        parent::before();

        $this->user = User::instance();
        if (!$this->user->isSignedIn()) {
            $this->redirect('sign', 302);
            return;
        }

        $id = (int) $this->request->param('id');
        $this->discipline = $this->generateDisciplineInfo($id);
        if ($this->discipline->authorID != $this->user['TeacherID'])
            throw HTTP_Exception::factory(403, "Недостаточно прав для редактирования.");

        $section = $this->request->param('section');
        $this->twig = Twig::factory(static::twigSource . $section);
        $this->twig->Discipline = $this->discipline;
        $this->twig->User = $this->user;
    }

    public function after() {
        $this->response->body($this->twig);
    }


    public function action_edit_settings() {
        $this->twig->SubjectsList = Model_Subjects::byFaculty($this->user->FacultyID);
        $this->twig->GradesList   = Model_Grades::toArray();
    }

    public function action_edit_structure() {
        $map = Model_Map::getMapForDiscipline($this->discipline->ID);
        $this->twig->Map = $this->getMapInfo($map);
    }
    
    public function action_edit_groups() {
        $this->twig->set([
            'GroupsForDiscipline' => $this->discipline->getGroups(),
            'Groups'              => Model_Faculty::with($this->discipline->facultyID)
                                        ->getGroups($this->discipline->gradeID),
        ]);
    }
    
    public function action_edit_students() {
        $this->manageGroups($this->discipline, $students, $attached);

        $this->twig->set([
            'Groups'         => $students,
            'GroupsAttached' => $attached,
            'GradesList'     => Model_Grades::toArray(),
            'GroupsList'     => Model_Faculty::with($this->discipline->facultyID)
                                    ->getGroups($this->discipline->gradeID),
        ]);
    }
    
    public function action_edit_teachers() {
        $this->twig->set([
            'FacultiesList'    => Model_Faculties::load(),
            'Departments'      => Model_Departments::byFaculty($this->user->FacultyID),
            'Author'           => Model_Account::getPersonalInfo($this->discipline->authorID),
            'BindTeachersList' => Model_Teachers::getNamesForDiscipline($this->discipline->ID),
        ]);
    }


    private function getMapInfo($map) {
        $maxRate = $i = $module = 0;
        $mapHandled = [];

        foreach ($map as $row) {
            if ($row['ModuleType'] == 'extra') {
                continue;
            }

            if ($row['ModuleID'] != $module) {
                $i++;
                $module = $row['ModuleID'];
                $mapHandled[$i]['ModuleID'] = $row['ModuleID'];
                $mapHandled[$i]['ModuleType'] = $row['ModuleType'];
                if ($row['ModuleType'] == 'bonus') {
                    $mapHandled['isSetBonus'] = true;
                }
                $mapHandled[$i]['CurrentControl'] = 0;
                $mapHandled[$i]['LandmarkControl'] = 0;
            }

            if (!isset($mapHandled[$i]['SubmodulesCount'])) {
                $mapHandled[$i]['SubmodulesCount'] = 0;
                $mapHandled[$i]['MaxRate'] = 0;
            }

            $mapHandled[$i]['IsExam'] = $row['IsExam'];
            $j = $mapHandled[$i]['SubmodulesCount'] += 1;
            $mapHandled[$i]['MaxRate'] += (int)$row['MaxRate'];
            $mapHandled[$i]['ModuleTitle'] = $row['ModuleName'];
            $mapHandled[$i][$j]['SubmoduleID'] = $row['SubmoduleID'];
            $mapHandled[$i][$j]['Title'] = $row['SubmoduleName'];
            $mapHandled[$i][$j]['Description'] = $row['SubmoduleDescription'];
            $mapHandled[$i][$j]['SubmoduleControl'] = $row['SubmoduleType'];
            $mapHandled[$i][$j]['MaxRate'] = (int)$row['MaxRate'];
            $maxRate += $row['MaxRate'];

            if ($row['SubmoduleType'] == 'CurrentControl') {
                $mapHandled[$i]['CurrentControl'] += (int)$row['MaxRate'];
            } elseif ($row['SubmoduleType'] == 'LandmarkControl') {
                $mapHandled[$i]['LandmarkControl'] += (int)$row['MaxRate'];
            }
        }

        $mapHandled['ModulesCount'] = $i;
        $mapHandled['MaxRate'] = (int)$maxRate;
        if ($mapHandled['isSetBonus'] == true) {
            $mapHandled['MaxRate'] = $mapHandled['MaxRate'] - 10;
        }

        return $mapHandled;
    }

    private function manageGroups(Model_Discipline $discipline, &$std, &$attached) {
        $students = Model_Students::ofDiscipline($discipline->ID);
        $groupID = $i = $k = $n = $j = 0;
        $attached = $std = [];

        foreach ($students as $row) {
            if ($row['GroupID'] != $groupID) {
                $groupID = $row['GroupID'];

                $title = [
                    'GroupID'  => $row['GroupID'],
                    'GradeID'  => $row['GradeID'],
                    'GradeNum' => $row['GradeNum'],
                    'GroupNum' => $row['GroupNum'],
                    'Degree'   => Model_Grades::getDegreeTitle($row['Degree']),
                ];

                if ($row['AttachType'] == 'attach') {
                    $attached[++$k] = $title;
                } else {
                    $std[++$i] = $title;
                }
            }

            $list = [
                'LastName'   => $row['LastName'],
                'FirstName'  => $row['FirstName'],
                'SecondName' => $row['SecondName'],
                'AttachType' => $row['AttachType'],
                'ID' => $row['ID'],
            ];

            if ($row['AttachType'] == 'attach') {
                $attached[$k]['students'][] = $list;
            } else {
                $std[$i]['students'][] = $list;
            }
        }
    }

    private function generateDisciplineInfo($id) {
        $discipline = Model_Discipline::load($id);
        $semester = Model_Semesters::getInfo($discipline->semesterID);
        $names = Model_Teachers::getNamesForDiscipline($discipline->ID, $concat = true);

        $discipline['teachers']      = implode(', ', $names);
        $discipline['semesterNum']   = $semester['Num'] == 1 ? 'Осенний' : 'Весенний';
        $discipline['semesterYear']  = $semester['Year'];
        // todo: link to discipline's work plan & department name

        return $discipline;
    }
}
            